# zincdl/defaults.py


DEFAULTS = {
    "subset": None,
    "mw": 250,
    "logp": 0,
    "reactivity": "standard",
    "purchasability": "in-stock",
    "ph": "ref",
    "charge": 0,
    "reac_exclusive": True,
    "purch_exclusive": True,
    "fmt": "smi"
}